import base64
import json
import os
import subprocess
import sys

from charmhelpers.core import hookenv, host
from charmhelpers.core.templating import render
from charmhelpers.core.services.base import ManagerCallback
from charmhelpers.fetch import apt_install, apt_update, add_source


CONFIG_FILE = '/etc/telegraf/telegraf.conf'

CONFIG_DIR = '/etc/telegraf/telegraf.d'


class Restart(ManagerCallback):
    """Restart the service, only if the config changed"""

    def _update_persisted_data(self, key, value):
        """Persist the given 'value' for the given 'key' and return the old value.

        @param key: The key to update.
        @param value: The value to persist.
        @return: The old value of the key, or None if it's a new value.
        """
        config = hookenv.config()
        old = config.get(key, None)
        config[key] = value
        config.save()
        return old

    def __call__(self, manager, service_name, event_name):
        if hookenv.hook_name().endswith('relation-changed'):
            rel_type = hookenv.relation_type()
            config_path = os.path.join('{}/{}.conf'.format(CONFIG_DIR, rel_type))
            new_hash = host.file_hash(config_path)
            old_hash = self._update_persisted_data(rel_type, new_hash)
            if new_hash != old_hash:
                hookenv.log('Restarting service, {}'.format(hookenv.hook_name()))
                host.service_restart(service_name)
                return
        elif hookenv.hook_name() == 'config-changed':
            new_hash = host.file_hash(CONFIG_FILE)
            old_hash = self._update_persisted_data('config_hash', new_hash)
            if new_hash != old_hash:
                hookenv.log('Restarting service, config file changed')
                host.service_restart(service_name)
                return
        hookenv.log('Not restarting service, config and relations unchanged')


def install():
    hookenv.log('Installing telegraf')
    config = hookenv.config()
    if config['package_name']:
        pkg_file = os.path.join(hookenv.charm_dir(), "files",
                                config['package_name'])
        if os.path.exists(pkg_file):
            if subprocess.call(["dpkg", "-i", pkg_file]) != 0:
                hookenv.log("CHARM: Error installing telegraf package")
                sys.exit(1)
        else:
            if config['apt_repository'] and config['apt_repository_key']:
                add_source(config['apt_repository'],
                           config['apt_repository_key'])
                apt_update()
            apt_install(config['package_name'], options=['--force-yes'])


def log_start(service_name):
    hookenv.log('telegraf starting')


def render_config(service_name):
    config = hookenv.config()
    context = config.copy()
    inputs = base64.b64decode(config['inputs_config'])
    outputs = base64.b64decode(config['outputs_config'])
    tags = []
    if config['tags']:
        for tag in config['tags'].split(','):
            key, value = tag.split("=")
            tags.append('{} = "{}"'.format(key, value))
    context["tags"] = tags
    if inputs:
        context["inputs"] = inputs
    else:
        # use base inputs from charm templates
        with open('templates/base_inputs.conf', 'r') as fd:
            context["inputs"] = fd.read()
    if outputs:
        context["outputs"] = outputs
    else:
        context["outputs"] = ""
        hookenv.log("No output plugins in main config.")
    hookenv.log("Updating config file")
    render(source='telegraf.conf.tmpl', target=CONFIG_FILE, context=context)


def elasticsearch_input(service_name):
    template = """
[elasticsearch]
  servers = %s
  local = true
  cluster_health = true
"""
    rels = hookenv.relations_of_type('elasticsearch')
    hosts = []
    inputs = []
    for rel in rels:
        es_host = rel.get('host')
        port = rel.get('port')
        if not es_host or not port:
            hookenv.log('No host received for relation: {}.'.format(rel))
            continue
        hosts.append("http://{}:{}".format(es_host, port))
    config_path = '{}/{}.conf'.format(CONFIG_DIR, 'elasticsearch')
    if hosts:
        inputs.append(template % json.dumps(hosts))
        hookenv.log("Updating {} plugin config file".format('elasticsearch'))
        host.write_file(config_path, '\n'.join(inputs))
    elif os.path.exists(config_path):
        os.unlink(config_path)


def mongodb_input(service_name):
    template = """
[[inputs.mongodb]]
  servers = %s
"""
    rels = hookenv.relations_of_type('mongodb')
    mongo_addresses = []
    inputs = []
    for rel in rels:
        addr = rel['private-address']
        port = rel.get('port', None)
        if port:
            mongo_address = '%s:%s' % (addr, port)
        else:
            mongo_address = addr
        mongo_addresses.append(mongo_address)
    config_path = '{}/{}.conf'.format(CONFIG_DIR, 'mongodb')
    if mongo_addresses:
        inputs.append(template % json.dumps(mongo_addresses))
        hookenv.log("Updating {} plugin config file".format('mongodb'))
        host.write_file(config_path, '\n'.join(inputs))
    elif os.path.exists(config_path):
        os.unlink(config_path)


def postgresql_input(service_name):
    template = """
[[inputs.postgresql]]
  address = "host=%(host)s user=%(user)s password=%(password)s dbname=%(database)s"
"""
    required_keys = ['host', 'user', 'password', 'database']
    rels = hookenv.relations_of_type('postgresql')
    inputs = []
    for rel in rels:
        if all([rel.get(key) for key in required_keys]) \
                and hookenv.local_unit() in rel.get('allowed-units'):
            inputs.append(template % rel)
    config_path = '{}/{}.conf'.format(CONFIG_DIR, 'postgresql')
    if inputs:
        hookenv.log("Updating {} plugin config file".format('postgresql'))
        host.write_file(config_path, '\n'.join(inputs))
    elif os.path.exists(config_path):
        os.unlink(config_path)


def influxdb_api_output(service_name):
    required_keys = ['hostname', 'port', 'user', 'password']
    rels = hookenv.relations_of_type('influxdb-api')
    endpoints = []
    user = None
    password = None
    for rel in rels:
        if all([rel.get(key) for key in required_keys]):
            endpoints.append("http://{}:{}".format(rel['hostname'], rel['port']))
            if user is None:
                user = rel['user']
            if password is None:
                password = rel['password']
    config_path = '{}/{}.conf'.format(CONFIG_DIR, 'influxdb-api')
    if endpoints:
        hookenv.log("Updating {} plugin config file".format('postgresql'))
        render(source='influxdb-api.conf.tmpl', target=config_path,
               context={'urls': json.dumps(endpoints),
                        'username': '{}'.format(user),
                        'password': '{}'.format(password)})
    elif os.path.exists(config_path):
        os.unlink(config_path)
